@@ -71,11 +71,11 @@ public:
     QOpenGLTexture *getTexture();
     int iviId() const { return m_iviId; }
 
-    QRect globalGeometry() const { return QRect(globalPosition(), surface()->size()); }
+    QRect globalGeometry() const { return QRect(globalPosition(), surface()->destinationSize()); }
     void setGlobalPosition(const QPoint &globalPos) { m_pos = globalPos; m_positionSet = true; }
     QPoint globalPosition() const { return m_pos; }
     QPoint mapToLocal(const QPoint &globalPos) const;
-    QSize size() const { return surface() ? surface()->size() : QSize(); }
+    QSize size() const { return surface() ? surface()->destinationSize() : QSize(); }
 
     void initPosition(const QSize &screenSize, const QSize &surfaceSize);
 
@@ -102,7 +102,7 @@ public:
     void handleMousePress(const QPoint &position, Qt::MouseButton button);
     void handleMouseRelease(const QPoint &position, Qt::MouseButton button, Qt::MouseButtons buttons);
     void handleMouseMove(const QPoint &position);
-    void handleMouseWheel(Qt::Orientation orientation, int delta);
+    void handleMouseWheel(const QPoint &angleDelta);
 
     void handleKeyPress(quint32 nativeScanCode);
     void handleKeyRelease(quint32 nativeScanCode);
