@@ -93,9 +93,12 @@ void QWaylandTextInput::reset()
 void QWaylandTextInput::commit()
 {
     if (QObject *o = QGuiApplication::focusObject()) {
-        QInputMethodEvent event;
-        event.setCommitString(m_preeditCommit);
-        QCoreApplication::sendEvent(o, &event);
+        if (!m_preeditCommit.isEmpty()) {
+            QInputMethodEvent event;
+            event.setCommitString(m_preeditCommit);
+            m_preeditCommit = QString();
+            QCoreApplication::sendEvent(o, &event);
+        }
     }
 
     reset();
@@ -665,3 +668,5 @@ bool QWaylandInputContext::filterEvent(const QEvent *event)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qwaylandinputcontext_p.cpp"
