@@ -6,7 +6,7 @@
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
@@ -15,24 +15,14 @@
 ** and conditions see https://www.qt.io/terms-conditions. For further
 ** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** General Public License version 3 or (at your option) any later version
+** approved by the KDE Free Qt Foundation. The licenses are as published by
+** the Free Software Foundation and appearing in the file LICENSE.GPL3
 ** included in the packaging of this file. Please review the following
 ** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -72,7 +62,6 @@
 
 #include "extensions/qwaylandqtwindowmanager.h"
 
-#include "qwaylandxkb_p.h"
 #include "qwaylandsharedmemoryformathelper_p.h"
 
 #include <QtCore/QCoreApplication>
@@ -125,16 +114,25 @@ public:
 
         QWaylandKeyboardPrivate *keyb = QWaylandKeyboardPrivate::get(seat->keyboard());
 
+#if defined(Q_OS_QNX)
+        // The QNX platform plugin delivers scan codes that haven't been adjusted to be
+        // xkbcommon compatible. xkbcommon requires that the scan codes be bumped up by
+        // 8 because that's how evdev/XKB deliver scan codes. You might think that it
+        // would've been better to remove this (odd) requirement from xkbcommon on QNX
+        // but it turns out that conforming to it has much less impact.
+        static int offset = QGuiApplication::platformName() == QStringLiteral("qnx") ? 8 : 0;
+        ke->nativeScanCode += offset;
+#endif
         uint32_t code = ke->nativeScanCode;
         bool isDown = ke->keyType == QEvent::KeyPress;
 
 #if QT_CONFIG(xkbcommon)
-        QString text;
-        Qt::KeyboardModifiers modifiers = QWaylandXkb::modifiers(keyb->xkbState());
+        xkb_state *xkbState = keyb->xkbState();
+        Qt::KeyboardModifiers modifiers = QXkbCommon::modifiers(xkbState);
 
-        const xkb_keysym_t sym = xkb_state_key_get_one_sym(keyb->xkbState(), code);
-        int qtkey;
-        std::tie(qtkey, text) = QWaylandXkb::keysymToQtKey(sym, modifiers);
+        const xkb_keysym_t sym = xkb_state_key_get_one_sym(xkbState, code);
+        int qtkey = QXkbCommon::keysymToQtKey(sym, modifiers, xkbState, code);
+        QString text = QXkbCommon::lookupString(xkbState, code);
 
         ke->key = qtkey;
         ke->modifiers = modifiers;
@@ -162,12 +160,24 @@ QWaylandCompositorPrivate::QWaylandCompositorPrivate(QWaylandCompositor *composi
 {
     if (QGuiApplication::platformNativeInterface())
         display = static_cast<wl_display*>(QGuiApplication::platformNativeInterface()->nativeResourceForIntegration("server_wl_display"));
-    if (!display)
+
+    if (!display) {
         display = wl_display_create();
+        ownsDisplay = true;
+    }
+
     eventHandler.reset(new QtWayland::WindowSystemEventHandler(compositor));
     timer.start();
 
     QWindowSystemInterfacePrivate::installWindowSystemEventHandler(eventHandler.data());
+
+#if QT_CONFIG(xkbcommon)
+    mXkbContext.reset(xkb_context_new(XKB_CONTEXT_NO_FLAGS));
+    if (!mXkbContext) {
+        qWarning("Failed to create a XKB context: keymap will not be supported");
+        return;
+    }
+#endif
 }
 
 void QWaylandCompositorPrivate::init()
@@ -189,17 +199,17 @@ void QWaylandCompositorPrivate::init()
     buffer_manager = new QtWayland::BufferManager(q);
 
     wl_display_init_shm(display);
-    QVector<wl_shm_format> formats = QWaylandSharedMemoryFormatHelper::supportedWaylandFormats();
-    foreach (wl_shm_format format, formats)
+    const QVector<wl_shm_format> formats = QWaylandSharedMemoryFormatHelper::supportedWaylandFormats();
+    for (wl_shm_format format : formats)
         wl_display_add_shm_format(display, format);
 
     if (!socket_name.isEmpty()) {
         if (wl_display_add_socket(display, socket_name.constData()))
-            qFatal("Fatal: Failed to open server socket\n");
+            qFatal("Fatal: Failed to open server socket: \"%s\". XDG_RUNTIME_DIR is: \"%s\"\n", socket_name.constData(), getenv("XDG_RUNTIME_DIR"));
     } else {
         const char *autoSocketName = wl_display_add_socket_auto(display);
         if (!autoSocketName)
-            qFatal("Fatal: Failed to open server socket\n");
+            qFatal("Fatal: Failed to open default server socket. XDG_RUNTIME_DIR is: \"%s\"\n", getenv("XDG_RUNTIME_DIR"));
         socket_name = autoSocketName;
         emit q->socketNameChanged(socket_name);
     }
@@ -213,7 +223,7 @@ void QWaylandCompositorPrivate::init()
     int fd = wl_event_loop_get_fd(loop);
 
     QSocketNotifier *sockNot = new QSocketNotifier(fd, QSocketNotifier::Read, q);
-    QObject::connect(sockNot, SIGNAL(activated(int)), q, SLOT(processWaylandEvents()));
+    QObject::connect(sockNot, SIGNAL(activated(QSocketDescriptor)), q, SLOT(processWaylandEvents()));
 
     QAbstractEventDispatcher *dispatcher = QGuiApplicationPrivate::eventDispatcher;
     QObject::connect(dispatcher, SIGNAL(aboutToBlock()), q, SLOT(processWaylandEvents()));
@@ -223,7 +233,7 @@ void QWaylandCompositorPrivate::init()
 
     initialized = true;
 
-    Q_FOREACH (QPointer<QObject> object, polish_objects) {
+    for (const QPointer<QObject> &object : qExchange(polish_objects, {})) {
         if (object) {
             QEvent polishEvent(QEvent::Polish);
             QCoreApplication::sendEvent(object.data(), &polishEvent);
@@ -235,18 +245,24 @@ void QWaylandCompositorPrivate::init()
 
 QWaylandCompositorPrivate::~QWaylandCompositorPrivate()
 {
-    qDeleteAll(clients);
+    // Take copies, since the lists will get modified as elements are deleted
+    const auto clientsToDelete = clients;
+    qDeleteAll(clientsToDelete);
 
-    qDeleteAll(outputs);
+    const auto outputsToDelete = outputs;
+    qDeleteAll(outputsToDelete);
 
 #if QT_CONFIG(wayland_datadevice)
     delete data_device_manager;
 #endif
 
+#if QT_CONFIG(opengl)
     // Some client buffer integrations need to clean up before the destroying the wl_display
     client_buffer_integration.reset();
+#endif
 
-    wl_display_destroy(display);
+    if (ownsDisplay)
+        wl_display_destroy(display);
 }
 
 void QWaylandCompositorPrivate::preInit()
@@ -288,7 +304,7 @@ void QWaylandCompositorPrivate::addPolishObject(QObject *object)
     if (initialized) {
         QCoreApplication::postEvent(object, new QEvent(QEvent::Polish));
     } else {
-        polish_objects.append(object);
+        polish_objects.push_back(object);
     }
 }
 
@@ -364,8 +380,6 @@ void QWaylandCompositorPrivate::initializeHardwareIntegration()
 
     if (client_buffer_integration)
         client_buffer_integration->initializeHardware(display);
-    if (server_buffer_integration)
-        server_buffer_integration->initializeHardware(q);
 #endif
 }
 
@@ -407,6 +421,7 @@ void QWaylandCompositorPrivate::loadClientBufferIntegration()
 void QWaylandCompositorPrivate::loadServerBufferIntegration()
 {
 #if QT_CONFIG(opengl)
+    Q_Q(QWaylandCompositor);
     QStringList keys = QtWayland::ServerBufferIntegrationFactory::keys();
     QString targetKey;
     QByteArray serverBufferIntegration = qgetenv("QT_WAYLAND_SERVER_BUFFER_INTEGRATION");
@@ -415,14 +430,28 @@ void QWaylandCompositorPrivate::loadServerBufferIntegration()
     }
     if (!targetKey.isEmpty()) {
         server_buffer_integration.reset(QtWayland::ServerBufferIntegrationFactory::create(targetKey, QStringList()));
-        if (hw_integration)
-            hw_integration->setServerBufferIntegration(targetKey);
+        if (server_buffer_integration) {
+            qCDebug(qLcWaylandCompositorHardwareIntegration)
+                    << "Loaded server buffer integration:" << targetKey;
+            if (!server_buffer_integration->initializeHardware(q)) {
+                qCWarning(qLcWaylandCompositorHardwareIntegration)
+                        << "Failed to initialize hardware for server buffer integration:" << targetKey;
+                server_buffer_integration.reset();
+            }
+        } else {
+            qCWarning(qLcWaylandCompositorHardwareIntegration)
+                    << "Failed to load server buffer integration:" << targetKey;
+        }
     }
+
+    if (server_buffer_integration && hw_integration)
+        hw_integration->setServerBufferIntegration(targetKey);
 #endif
 }
 
 /*!
   \qmltype WaylandCompositor
+  \instantiates QWaylandCompositor
   \inqmlmodule QtWayland.Compositor
   \since 5.8
   \brief Manages the Wayland display server.
@@ -456,33 +485,35 @@ void QWaylandCompositorPrivate::loadServerBufferIntegration()
 /*!
   \qmlsignal void QtWaylandCompositor::WaylandCompositor::surfaceRequested(WaylandClient client, int id, int version)
 
-  This signal is emitted when a client has created a surface.
-  The slot connecting to this signal may create and initialize
-  a WaylandSurface instance in the scope of the slot.
-  Otherwise a default surface is created.
+  This signal is emitted when a \a client has created a surface with id \a id.
+  The interface \a version is also available.
+
+  The slot connecting to this signal may create and initialize a WaylandSurface
+  instance in the scope of the slot. Otherwise a default surface is created.
 */
 
 /*!
   \fn void QWaylandCompositor::surfaceRequested(QWaylandClient *client, uint id, int version)
 
-  This signal is emitted when a client has created a surface.
-  The slot connecting to this signal may create and initialize
-  a QWaylandSurface instance in the scope of the slot.
-  Otherwise a default surface is created.
+  This signal is emitted when a \a client has created a surface with id \a id.
+  The interface \a version is also available.
+
+  The slot connecting to this signal may create and initialize a QWaylandSurface
+  instance in the scope of the slot. Otherwise a default surface is created.
 
   Connections to this signal must be of Qt::DirectConnection connection type.
 */
 
 /*!
-  \qmlsignal void QtWaylandCompositor::WaylandCompositor::surfaceCreated(QWaylandSurface *surface)
+  \qmlsignal void QtWaylandCompositor::WaylandCompositor::surfaceCreated(WaylandSurface surface)
 
-  This signal is emitted when a new WaylandSurface instance has been created.
+  This signal is emitted when a new WaylandSurface instance \a surface has been created.
 */
 
 /*!
   \fn void QWaylandCompositor::surfaceCreated(QWaylandSurface *surface)
 
-  This signal is emitted when a new QWaylandSurface instance has been created.
+  This signal is emitted when a new QWaylandSurface instance \a surface has been created.
 */
 
 /*!
@@ -586,7 +617,7 @@ QByteArray QWaylandCompositor::socketName() const
  * \qmlmethod QtWaylandCompositor::WaylandCompositor::addSocketDescriptor(fd)
  * \since 5.12
  *
- * Listen for client connections on a file descriptor referring to a
+ * Listen for client connections on a file descriptor, \a fd, referring to a
  * server socket already bound and listening.
  *
  * Does not take ownership of the file descriptor; it must be closed
@@ -598,7 +629,7 @@ QByteArray QWaylandCompositor::socketName() const
  */
 
 /*!
- * Listen for client connections on a file descriptor referring to a
+ * Listen for client connections on a file descriptor, \a fd, referring to a
  * server socket already bound and listening.
  *
  * Does not take ownership of the file descriptor; it must be closed
@@ -692,7 +723,7 @@ QList<QWaylandSurface *> QWaylandCompositor::surfacesForClient(QWaylandClient* c
 {
     Q_D(const QWaylandCompositor);
     QList<QWaylandSurface *> surfs;
-    foreach (QWaylandSurface *surface, d->all_surfaces) {
+    for (QWaylandSurface *surface : d->all_surfaces) {
         if (surface->client() == client)
             surfs.append(surface);
     }
@@ -714,7 +745,7 @@ QList<QWaylandSurface *> QWaylandCompositor::surfaces() const
 QWaylandOutput *QWaylandCompositor::outputFor(QWindow *window) const
 {
     Q_D(const QWaylandCompositor);
-    foreach (QWaylandOutput *output, d->outputs) {
+    for (QWaylandOutput *output : d->outputs) {
         if (output->window() == window)
             return output;
     }
