@@ -6,7 +6,7 @@
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
@@ -15,24 +15,14 @@
 ** and conditions see https://www.qt.io/terms-conditions. For further
 ** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** General Public License version 3 or (at your option) any later version
+** approved by the KDE Free Qt Foundation. The licenses are as published by
+** the Free Software Foundation and appearing in the file LICENSE.GPL3
 ** included in the packaging of this file. Please review the following
 ** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -59,6 +49,7 @@
 #include <QtWaylandCompositor/private/qwaylandcompositor_p.h>
 #include <QtWaylandCompositor/private/qwaylandview_p.h>
 #include <QtWaylandCompositor/private/qwaylandseat_p.h>
+#include <QtWaylandCompositor/private/qwaylandutils_p.h>
 
 #include <QtCore/private/qobject_p.h>
 
@@ -136,9 +127,9 @@ QWaylandSurfacePrivate::~QWaylandSurfacePrivate()
 
     bufferRef = QWaylandBufferRef();
 
-    foreach (QtWayland::FrameCallback *c, pendingFrameCallbacks)
+    for (QtWayland::FrameCallback *c : qAsConst(pendingFrameCallbacks))
         c->destroy();
-    foreach (QtWayland::FrameCallback *c, frameCallbacks)
+    for (QtWayland::FrameCallback *c : qAsConst(frameCallbacks))
         c->destroy();
 }
 
@@ -151,7 +142,8 @@ void QWaylandSurfacePrivate::removeFrameCallback(QtWayland::FrameCallback *callb
 void QWaylandSurfacePrivate::notifyViewsAboutDestruction()
 {
     Q_Q(QWaylandSurface);
-    foreach (QWaylandView *view, views) {
+    const auto viewsCopy = views; // Views will be removed from the list when marked as destroyed
+    for (QWaylandView *view : viewsCopy) {
         QWaylandViewPrivate::get(view)->markSurfaceAsDestroyed(q);
     }
     if (hasContent) {
@@ -235,21 +227,29 @@ void QWaylandSurfacePrivate::surface_commit(Resource *)
 
     // Needed in order to know whether we want to emit signals later
     QSize oldBufferSize = bufferSize;
+    QRectF oldSourceGeometry = sourceGeometry;
+    QSize oldDestinationSize = destinationSize;
     bool oldHasContent = hasContent;
     int oldBufferScale = bufferScale;
 
     // Update all internal state
     if (pending.buffer.hasBuffer() || pending.newlyAttached)
         bufferRef = pending.buffer;
-    bufferSize = bufferRef.size();
-    damage = pending.damage.intersected(QRect(QPoint(), bufferSize));
-    hasContent = bufferRef.hasContent();
     bufferScale = pending.bufferScale;
+    bufferSize = bufferRef.size();
+    QSize surfaceSize = bufferSize / bufferScale;
+    sourceGeometry = !pending.sourceGeometry.isValid() ? QRect(QPoint(), surfaceSize) : pending.sourceGeometry;
+    destinationSize = pending.destinationSize.isEmpty() ? sourceGeometry.size().toSize() : pending.destinationSize;
+    damage = pending.damage.intersected(QRect(QPoint(), destinationSize));
+    hasContent = bufferRef.hasContent();
     frameCallbacks << pendingFrameCallbacks;
-    inputRegion = pending.inputRegion.intersected(QRect(QPoint(), bufferSize));
-    opaqueRegion = pending.opaqueRegion.intersected(QRect(QPoint(), bufferSize));
+    inputRegion = pending.inputRegion.intersected(QRect(QPoint(), destinationSize));
+    opaqueRegion = pending.opaqueRegion.intersected(QRect(QPoint(), destinationSize));
     QPoint offsetForNextFrame = pending.offset;
 
+    if (viewport)
+        viewport->checkCommittedState();
+
     // Clear per-commit state
     pending.buffer = QWaylandBufferRef();
     pending.offset = QPoint();
@@ -269,12 +269,25 @@ void QWaylandSurfacePrivate::surface_commit(Resource *)
 
     emit q->damaged(damage);
 
-    if (oldBufferSize != bufferSize)
+    if (oldBufferSize != bufferSize) {
+        emit q->bufferSizeChanged();
+#if QT_DEPRECATED_SINCE(5, 13)
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_DEPRECATED
         emit q->sizeChanged();
+QT_WARNING_POP
+#endif
+    }
 
     if (oldBufferScale != bufferScale)
         emit q->bufferScaleChanged();
 
+    if (oldDestinationSize != destinationSize)
+        emit q->destinationSizeChanged();
+
+    if (oldSourceGeometry != sourceGeometry)
+        emit q->sourceGeometryChanged();
+
     if (oldHasContent != hasContent)
         emit q->hasContentChanged();
 
@@ -322,6 +335,7 @@ QtWayland::ClientBuffer *QWaylandSurfacePrivate::getBuffer(struct ::wl_resource
 
 /*!
  * \qmltype WaylandSurface
+ * \instantiates QWaylandSurface
  * \inqmlmodule QtWayland.Compositor
  * \since 5.8
  * \brief Represents a rectangular area on an output device.
@@ -372,7 +386,8 @@ QWaylandSurface::QWaylandSurface(QWaylandSurfacePrivate &dptr)
 QWaylandSurface::~QWaylandSurface()
 {
     Q_D(QWaylandSurface);
-    QWaylandCompositorPrivate::get(d->compositor)->unregisterSurface(this);
+    if (d->compositor)
+        QWaylandCompositorPrivate::get(d->compositor)->unregisterSurface(this);
     d->notifyViewsAboutDestruction();
 }
 
@@ -432,9 +447,7 @@ QWaylandClient *QWaylandSurface::client() const
 }
 
 /*!
- * \property QWaylandSurface::waylandClient
- *
- * This property holds the \c wl_client using this QWaylandSurface.
+ * Holds the \c wl_client using this QWaylandSurface.
  */
 ::wl_client *QWaylandSurface::waylandClient() const
 {
@@ -462,22 +475,110 @@ bool QWaylandSurface::hasContent() const
 }
 
 /*!
- * \qmlproperty size QtWaylandCompositor::WaylandSurface::size
+ * \qmlproperty rect QtWaylandCompositor::WaylandSurface::sourceGeometry
+ * \since 5.13
  *
- * This property holds the WaylandSurface's size in pixels.
+ * This property describes the portion of the attached Wayland buffer that should
+ * be drawn on the screen. The coordinates are from the corner of the buffer and are
+ * scaled by \l bufferScale.
+ *
+ * \sa bufferScale
+ * \sa bufferSize
+ * \sa destinationSize
  */
 
 /*!
- * \property QWaylandSurface::size
+ * \property QWaylandSurface::sourceGeometry
+ * \since 5.13
  *
- * This property holds the QWaylandSurface's size in pixels.
+ * This property describes the portion of the attached QWaylandBuffer that should
+ * be drawn on the screen. The coordinates are from the corner of the buffer and are
+ * scaled by \l bufferScale.
+ *
+ * \sa bufferScale
+ * \sa bufferSize
+ * \sa destinationSize
  */
-QSize QWaylandSurface::size() const
+QRectF QWaylandSurface::sourceGeometry() const
+{
+    Q_D(const QWaylandSurface);
+    return d->sourceGeometry;
+}
+
+/*!
+ * \qmlproperty size QtWaylandCompositor::WaylandSurface::destinationSize
+ * \since 5.13
+ *
+ * This property holds the size of this WaylandSurface in surface coordinates.
+ *
+ * \sa bufferScale
+ * \sa bufferSize
+ */
+
+/*!
+ * \property QWaylandSurface::destinationSize
+ * \since 5.13
+ *
+ * This property holds the size of this WaylandSurface in surface coordinates.
+ *
+ * \sa bufferScale
+ * \sa bufferSize
+ */
+QSize QWaylandSurface::destinationSize() const
+{
+    Q_D(const QWaylandSurface);
+    return d->destinationSize;
+}
+
+/*!
+ * \qmlproperty size QtWaylandCompositor::WaylandSurface::bufferSize
+ *
+ * This property holds the size of the current buffer of this WaylandSurface in pixels,
+ * not in surface coordinates.
+ *
+ * For the size in surface coordinates, use \l destinationSize instead.
+ *
+ * \sa destinationSize
+ * \sa bufferScale
+ */
+
+/*!
+ * \property QWaylandSurface::bufferSize
+ *
+ * This property holds the size of the current buffer of this QWaylandSurface in pixels,
+ * not in surface coordinates.
+ *
+ * For the size in surface coordinates, use \l destinationSize instead.
+ *
+ * \sa destinationSize
+ * \sa bufferScale
+ */
+QSize QWaylandSurface::bufferSize() const
 {
     Q_D(const QWaylandSurface);
     return d->bufferSize;
 }
 
+#if QT_DEPRECATED_SINCE(5, 13)
+/*!
+ * \qmlproperty size QtWaylandCompositor::WaylandSurface::size
+ * \obsolete use bufferSize or destinationSize instead
+ *
+ * This property has been deprecated, use \l bufferSize or \l destinationSize instead.
+ */
+
+/*!
+ * \property QWaylandSurface::size
+ * \obsolete use bufferSize or destinationSize instead
+ *
+ * This property has been deprecated, use \l bufferSize or \l destinationSize instead.
+ */
+QSize QWaylandSurface::size() const
+{
+    return bufferSize();
+}
+#endif
+
 /*!
  * \qmlproperty size QtWaylandCompositor::WaylandSurface::bufferScale
  *
@@ -569,7 +670,7 @@ QWaylandCompositor *QWaylandSurface::compositor() const
 void QWaylandSurface::frameStarted()
 {
     Q_D(QWaylandSurface);
-    foreach (QtWayland::FrameCallback *c, d->frameCallbacks)
+    for (QtWayland::FrameCallback *c : qAsConst(d->frameCallbacks))
         c->canSend = true;
 }
 
@@ -602,10 +703,11 @@ bool QWaylandSurface::inputRegionContains(const QPoint &p) const
     return d->inputRegion.contains(p);
 }
 
-//TODO: Add appropriate \since version when this is made public.
 /*!
  * Returns \c true if the QWaylandSurface's input region contains the point \a position.
  * Otherwise returns \c false.
+ *
+ * \since 5.14
  */
 bool QWaylandSurface::inputRegionContains(const QPointF &position) const
 {
@@ -676,6 +778,31 @@ bool QWaylandSurface::isCursorSurface() const
     return d->isCursorSurface;
 }
 
+/*!
+ * \qmlproperty bool QtWaylandCompositor::WaylandSurface::inhibitsIdle
+ * \since 5.14
+ *
+ * This property holds whether this surface is intended to inhibit the idle
+ * behavior of the compositor such as screen blanking, locking and screen saving.
+ *
+ * \sa IdleInhibitManagerV1
+ */
+
+/*!
+ * \property QWaylandSurface::inhibitsIdle
+ * \since 5.14
+ *
+ * This property holds whether this surface is intended to inhibit the idle
+ * behavior of the compositor such as screen blanking, locking and screen saving.
+ *
+ * \sa QWaylandIdleInhibitManagerV1
+ */
+bool QWaylandSurface::inhibitsIdle() const
+{
+    Q_D(const QWaylandSurface);
+    return !d->idleInhibitors.isEmpty();
+}
+
 #if QT_CONFIG(im)
 QWaylandInputMethodControl *QWaylandSurface::inputMethodControl() const
 {
@@ -758,12 +885,12 @@ QList<QWaylandView *> QWaylandSurface::views() const
 }
 
 /*!
- * Returns the QWaylandSurface corresponding to the Wayland resource \a res.
+ * Returns the QWaylandSurface corresponding to the Wayland resource \a resource.
  */
-QWaylandSurface *QWaylandSurface::fromResource(::wl_resource *res)
+QWaylandSurface *QWaylandSurface::fromResource(::wl_resource *resource)
 {
-    if (auto *r = QWaylandSurfacePrivate::Resource::fromResource(res))
-        return static_cast<QWaylandSurfacePrivate *>(r->surface_object)->q_func();
+    if (auto p = QtWayland::fromResource<QWaylandSurfacePrivate *>(resource))
+        return p->q_func();
     return nullptr;
 }
 
@@ -777,11 +904,12 @@ struct wl_resource *QWaylandSurface::resource() const
 }
 
 /*!
- * Sets a \a role on the surface. A role defines how a surface will be mapped on screen, without a role
- * a surface is supposed to be hidden. Only one role at all times can be set on a surface. Although
+ * Sets a \a role on the surface. A role defines how a surface will be mapped on screen; without a role
+ * a surface is supposed to be hidden. Only one role can be set on a surface, at all times. Although
  * setting the same role many times is allowed, attempting to change the role of a surface will trigger
  * a protocol error to the \a errorResource and send an \a errorCode to the client.
  *
+ * Returns true if a role can be assigned; false otherwise.
  */
 bool QWaylandSurface::setRole(QWaylandSurfaceRole *role, wl_resource *errorResource, uint32_t errorCode)
 {
@@ -922,4 +1050,23 @@ void QWaylandSurfacePrivate::Subsurface::subsurface_set_desync(wl_subsurface::Re
  * This signal is emitted when a \a drag has started from this surface.
  */
 
+/*!
+ * \fn void damaged(const QRegion &rect)
+ *
+ * This signal is emitted when the client tells the compositor that a particular part of, or
+ * possibly the entire surface has been updated, so the compositor can redraw that part.
+ *
+ * While the compositor APIs take care of redrawing automatically, this function may be useful
+ * if you require a specific, custom behavior.
+ */
+
+/*!
+ * \fn void parentChanged(QWaylandSurface *newParent, QWaylandSurface *oldParent)
+ *
+ * This signal is emitted when the client has requested that this surface should be a
+ * subsurface of \a newParent.
+ */
+
 QT_END_NAMESPACE
+
+#include "moc_qwaylandsurface.cpp"
