@@ -82,3 +82,9 @@
  *
  * This property holds the window type of the QWaylandShellSurface.
  */
+
+QT_BEGIN_NAMESPACE
+
+QT_END_NAMESPACE
+
+#include "moc_qwaylandshellsurface.cpp"
