@@ -5,7 +5,7 @@
 **
 ** This file is part of the QtWaylandCompositor module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
@@ -14,24 +14,14 @@
 ** and conditions see https://www.qt.io/terms-conditions. For further
 ** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** General Public License version 3 or (at your option) any later version
+** approved by the KDE Free Qt Foundation. The licenses are as published by
+** the Free Software Foundation and appearing in the file LICENSE.GPL3
 ** included in the packaging of this file. Please review the following
 ** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -43,6 +33,7 @@
 #if QT_CONFIG(wayland_compositor_quick)
 #include "qwaylandxdgshellv5integration_p.h"
 #endif
+#include <QtWaylandCompositor/private/qwaylandutils_p.h>
 
 #include <QtWaylandCompositor/QWaylandCompositor>
 #include <QtWaylandCompositor/QWaylandSurface>
@@ -56,6 +47,8 @@
 
 QT_BEGIN_NAMESPACE
 
+#if QT_DEPRECATED_SINCE(5, 15)
+
 QWaylandSurfaceRole QWaylandXdgSurfaceV5Private::s_role("xdg_surface");
 QWaylandSurfaceRole QWaylandXdgPopupV5Private::s_role("xdg_popup");
 
@@ -114,9 +107,9 @@ QWaylandXdgPopupV5 *QWaylandXdgShellV5Private::topmostPopupForClient(wl_client *
     return clientPopups.empty() ? nullptr : clientPopups.last();
 }
 
-QWaylandXdgSurfaceV5 *QWaylandXdgShellV5Private::xdgSurfaceFromSurface(QWaylandSurface *surface)
+QWaylandXdgSurfaceV5 *QWaylandXdgShellV5Private::xdgSurfaceFromSurface(QWaylandSurface *surface) const
 {
-    Q_FOREACH (QWaylandXdgSurfaceV5 *xdgSurface, m_xdgSurfaces) {
+    for (QWaylandXdgSurfaceV5 *xdgSurface : m_xdgSurfaces) {
         if (surface == xdgSurface->surface())
             return xdgSurface;
     }
@@ -249,7 +242,7 @@ QRect QWaylandXdgSurfaceV5Private::calculateFallbackWindowGeometry() const
 {
     // TODO: The unset window geometry should include subsurfaces as well, so this solution
     // won't work too well on those kinds of clients.
-    return QRect(QPoint(0, 0), m_surface->size() / m_surface->bufferScale());
+    return QRect(QPoint(), m_surface->destinationSize());
 }
 
 void QWaylandXdgSurfaceV5Private::updateFallbackWindowGeometry()
@@ -413,7 +406,7 @@ void QWaylandXdgSurfaceV5Private::xdg_surface_ack_configure(Resource *resource,
             break;
     }
 
-    QVector<uint> changedStates;
+    std::vector<uint> changedStates;
     std::set_symmetric_difference(
                 m_lastAckedConfigure.states.begin(), m_lastAckedConfigure.states.end(),
                 config.states.begin(), config.states.end(),
@@ -422,7 +415,7 @@ void QWaylandXdgSurfaceV5Private::xdg_surface_ack_configure(Resource *resource,
     m_lastAckedConfigure = config;
 
     if (!changedStates.empty()) {
-        Q_FOREACH (uint state, changedStates) {
+        for (uint state : changedStates) {
             switch (state) {
             case QWaylandXdgSurfaceV5::State::MaximizedState:
                 emit q->maximizedChanged();
@@ -502,6 +495,7 @@ void QWaylandXdgPopupV5Private::xdg_popup_destroy(Resource *resource)
  * \qmltype XdgShellV5
  * \inqmlmodule QtWayland.Compositor
  * \since 5.8
+ * \obsolete
  * \brief Provides an extension for desktop-style user interfaces.
  *
  * The XdgShellV5 extension provides a way to associate an XdgSurfaceV5
@@ -512,15 +506,17 @@ void QWaylandXdgPopupV5Private::xdg_popup_destroy(Resource *resource)
  *
  * To provide the functionality of the shell extension in a compositor, create
  * an instance of the XdgShellV5 component and add it as a child of the
- * compositor: \code
- * import QtWayland.Compositor 1.0
+ * compositor:
+ *
+ * \qml \QtMinorVersion
+ * import QtWayland.Compositor 1.\1
  *
  * WaylandCompositor {
  *     XdgShellV5 {
  *         // ...
  *     }
  * }
- * \endcode
+ * \endqml
  *
  * \deprecated
  */
@@ -529,6 +525,7 @@ void QWaylandXdgPopupV5Private::xdg_popup_destroy(Resource *resource)
  * \class QWaylandXdgShellV5
  * \inmodule QtWaylandCompositor
  * \since 5.8
+ * \obsolete
  * \brief The QWaylandXdgShellV5 class is an extension for desktop-style user interfaces.
  *
  * The QWaylandXdgShellV5 extension provides a way to associate a QWaylandXdgSurfaceV5 with
@@ -577,7 +574,7 @@ void QWaylandXdgShellV5::initialize()
 QWaylandClient *QWaylandXdgShellV5::popupClient() const
 {
     Q_D(const QWaylandXdgShellV5);
-    Q_FOREACH (QWaylandXdgPopupV5 *popup, d->m_xdgPopups) {
+    for (QWaylandXdgPopupV5 *popup : d->m_xdgPopups) {
         if (popup->surface()->hasContent())
             return popup->surface()->client();
     }
@@ -598,7 +595,7 @@ QByteArray QWaylandXdgShellV5::interfaceName()
 }
 
 /*!
- * \qmlmethod void QtWaylandCompositor::XdgShellV5::ping()
+ * \qmlmethod void QtWaylandCompositor::XdgShellV5::ping(WaylandClient client)
  *
  * Sends a ping event to the \a client. If the client replies to the event, the
  * pong signal will be emitted.
@@ -624,15 +621,19 @@ uint QWaylandXdgShellV5::ping(QWaylandClient *client)
     return serial;
 }
 
+// ### remove once QMap has rbegin()/rend()
+template <typename Iterator>
+std::reverse_iterator<Iterator> make_reverse(Iterator it)
+{
+    return std::reverse_iterator<Iterator>(std::move(it));
+}
+
 void QWaylandXdgShellV5::closeAllPopups()
 {
     Q_D(QWaylandXdgShellV5);
-    Q_FOREACH (struct wl_client *client, d->m_xdgPopups.keys()) {
-        QList<QWaylandXdgPopupV5 *> popups = d->m_xdgPopups.values(client);
-        std::reverse(popups.begin(), popups.end());
-        Q_FOREACH (QWaylandXdgPopupV5 *currentTopmostPopup, popups) {
-            currentTopmostPopup->sendPopupDone();
-        }
+    // Close pop-ups from top-most to bottom-most, lest we get protocol errors:
+    for (auto rit = make_reverse(d->m_xdgPopups.end()), rend = make_reverse(d->m_xdgPopups.begin()); rit != rend; ++rit) {
+        (*rit)->sendPopupDone();
     }
 }
 
@@ -759,6 +760,7 @@ void QWaylandXdgShellV5::handleFocusChanged(QWaylandSurface *newSurface, QWaylan
  * \qmltype XdgSurfaceV5
  * \inqmlmodule QtWayland.Compositor
  * \since 5.8
+ * \obsolete
  * \brief Provides a \c xdg_surface that offers desktop-style compositor-specific features to a surface.
  *
  * This type is part of the \l{XdgShellV5} extension and provides a way to extend
@@ -774,6 +776,7 @@ void QWaylandXdgShellV5::handleFocusChanged(QWaylandSurface *newSurface, QWaylan
  * \class QWaylandXdgSurfaceV5
  * \inmodule QtWaylandCompositor
  * \since 5.8
+ * \obsolete
  * \brief The QWaylandXdgSurfaceV5 class provides desktop-style compositor-specific features to an xdg surface.
  *
  * This class is part of the QWaylandXdgShellV5 extension and provides a way to
@@ -786,6 +789,36 @@ void QWaylandXdgShellV5::handleFocusChanged(QWaylandSurface *newSurface, QWaylan
  * \deprecated
  */
 
+/*!
+    \enum QWaylandXdgSurfaceV5::ResizeEdge
+
+    \value NoneEdge
+           No edge defined.
+    \value TopEdge
+           Top egde.
+    \value BottomEdge
+           Bottom edge.
+    \value LeftEdge
+           Left edge.
+    \value TopLeftEdge
+           Top-left edge.
+    \value BottomLeftEdge
+           Bottom-left edge.
+    \value RightEdge
+           Right edge.
+    \value TopRightEdge
+           Top-right edge.
+    \value BottomRightEdge
+           Bottom-right edge.
+ */
+
+/*!
+ * \fn QWaylandXdgSurfaceV5::setTopLevel()
+ *
+ * This signal is emitted when the parent surface is unset, effectively
+ * making the window top level.
+ */
+
 /*!
  * \qmlsignal QtWaylandCompositor::XdgSurfaceV5::setTopLevel()
  *
@@ -793,6 +826,13 @@ void QWaylandXdgShellV5::handleFocusChanged(QWaylandSurface *newSurface, QWaylan
  * making the window top level.
  */
 
+/*!
+ * \fn QWaylandXdgSurfaceV5::setTransient()
+ *
+ * This signal is emitted when the parent surface is set, effectively
+ * making the window transient.
+ */
+
 /*!
  * \qmlsignal QtWaylandCompositor::XdgSurfaceV5::setTransient()
  *
@@ -837,7 +877,7 @@ void QWaylandXdgSurfaceV5::initialize(QWaylandXdgShellV5 *xdgShell, QWaylandSurf
     d->init(resource.resource());
     setExtensionContainer(surface);
     d->m_windowGeometry = d->calculateFallbackWindowGeometry();
-    connect(surface, &QWaylandSurface::sizeChanged, this, &QWaylandXdgSurfaceV5::handleSurfaceSizeChanged);
+    connect(surface, &QWaylandSurface::destinationSizeChanged, this, &QWaylandXdgSurfaceV5::handleSurfaceSizeChanged);
     connect(surface, &QWaylandSurface::bufferScaleChanged, this, &QWaylandXdgSurfaceV5::handleBufferScaleChanged);
     emit shellChanged();
     emit surfaceChanged();
@@ -984,7 +1024,9 @@ void QWaylandXdgSurfaceV5::initialize()
 QList<int> QWaylandXdgSurfaceV5::statesAsInts() const
 {
    QList<int> list;
-   Q_FOREACH (uint state, states()) {
+   const auto s = states();
+   list.reserve(s.size());
+   for (auto state : s) {
        list << static_cast<int>(state);
    }
    return list;
@@ -1179,10 +1221,9 @@ QWaylandSurfaceRole *QWaylandXdgSurfaceV5::role()
  */
 QWaylandXdgSurfaceV5 *QWaylandXdgSurfaceV5::fromResource(wl_resource *resource)
 {
-    auto xsResource = QWaylandXdgSurfaceV5Private::Resource::fromResource(resource);
-    if (!xsResource)
-        return nullptr;
-    return static_cast<QWaylandXdgSurfaceV5Private *>(xsResource->xdg_surface_object)->q_func();
+    if (auto p = QtWayland::fromResource<QWaylandXdgSurfaceV5Private *>(resource))
+        return p->q_func();
+    return nullptr;
 }
 
 QSize QWaylandXdgSurfaceV5::sizeForResize(const QSizeF &size, const QPointF &delta,
@@ -1236,7 +1277,8 @@ uint QWaylandXdgSurfaceV5::sendConfigure(const QSize &size, const QVector<uint>
 uint QWaylandXdgSurfaceV5::sendConfigure(const QSize &size, const QVector<QWaylandXdgSurfaceV5::State> &states)
 {
     QVector<uint> asUints;
-    Q_FOREACH (QWaylandXdgSurfaceV5::State state, states) {
+    asUints.reserve(states.size());
+    for (QWaylandXdgSurfaceV5::State state : states) {
         asUints << state;
     }
     return sendConfigure(size, asUints);
@@ -1319,6 +1361,7 @@ QWaylandQuickShellIntegration *QWaylandXdgSurfaceV5::createIntegration(QWaylandQ
  * \qmltype XdgPopupV5
  * \inqmlmodule QtWayland.Compositor
  * \since 5.8
+ * \obsolete
  * \brief Provides a \c xdg_popup interface that implements popup features for the xdg-shell protocol.
  *
  * This type is part of the \l{XdgShellV5} extension and provides a way to extend
@@ -1334,6 +1377,7 @@ QWaylandQuickShellIntegration *QWaylandXdgSurfaceV5::createIntegration(QWaylandQ
  * \class QWaylandXdgPopupV5
  * \inmodule QtWaylandCompositor
  * \since 5.8
+ * \obsolete
  * \brief The QWaylandXdgPopupV5 class provides menus for an xdg surface.
  *
  * This class is part of the QWaylandXdgShellV5 extension and provides a way to
@@ -1497,10 +1541,9 @@ QWaylandSurfaceRole *QWaylandXdgPopupV5::role()
 
 QWaylandXdgPopupV5 *QWaylandXdgPopupV5::fromResource(wl_resource *resource)
 {
-    auto popupResource = QWaylandXdgPopupV5Private::Resource::fromResource(resource);
-    if (!popupResource)
-        return nullptr;
-    return static_cast<QWaylandXdgPopupV5Private *>(popupResource->xdg_popup_object)->q_func();
+    if (auto p = QtWayland::fromResource<QWaylandXdgPopupV5Private *>(resource))
+        return p->q_func();
+    return nullptr;
 }
 
 void QWaylandXdgPopupV5::sendPopupDone()
@@ -1516,4 +1559,6 @@ QWaylandQuickShellIntegration *QWaylandXdgPopupV5::createIntegration(QWaylandQui
 }
 #endif
 
+#endif // QT_DEPRECATED_SINCE(5, 15)
+
 QT_END_NAMESPACE
