@@ -63,6 +63,7 @@ class tst_seatv4 : public QObject, private SeatV4Compositor
 {
     Q_OBJECT
 private slots:
+    void init();
     void cleanup();
     void bindsToSeat();
     void keyboardKeyPress();
@@ -85,6 +86,12 @@ private slots:
 #endif
 };
 
+void tst_seatv4::init()
+{
+    // Remove the extra outputs to clean up for the next test
+    exec([&] { while (auto *o = output(1)) remove(o); });
+}
+
 void tst_seatv4::cleanup()
 {
     QTRY_VERIFY2(isClean(), qPrintable(dirtyMessage()));
@@ -135,7 +142,7 @@ void tst_seatv4::setsCursorOnEnter()
     window.show();
     QCOMPOSITOR_TRY_VERIFY(xdgSurface() && xdgSurface()->m_committedConfigureSerial);
 
-    exec([&] { pointer()->sendEnter(xdgSurface()->m_surface, {32, 32}); });
+    exec([&] { pointer()->sendEnter(xdgSurface()->m_surface, {24, 24}); });
     QCOMPOSITOR_TRY_VERIFY(cursorSurface());
 }
 
@@ -360,7 +367,7 @@ static bool supportsCursorSizes(const QVector<uint> &sizes)
 
 static uint defaultCursorSize() {
     const int xCursorSize = qEnvironmentVariableIntValue("XCURSOR_SIZE");
-    return xCursorSize > 0 ? uint(xCursorSize) : 32;
+    return xCursorSize > 0 ? uint(xCursorSize) : 24;
 }
 
 void tst_seatv4::scaledCursor()
